################################################################################################################################
#
# Description:	This constraint file applies to the dual QDR II+ SRAM interface to the FPGA.
#
# Rev History: 	08-28-2013:	Changed timing constraint to 450 MHz and corrected IOSTANDARDs for 1.8V HSTL.
#				05-12-2013:	Removed a placement constraint that produced a bitstream generation error in Vivado 2014.
#				09-04-2013:	Inital release.
#
################################################################################################################################

################################################ Timing Constraints ################################################

create_clock -name qdriip_sys_clk -period 2.22 [get_ports qdriip_sys_clk_p]
          
create_clock -name sram_a_capt_clk -period 2.22 [get_ports {sram_a_qdriip_cq}]
          
create_clock -name sram_b_capt_clk -period 2.22 [get_ports {sram_b_qdriip_cq}]
          
# Note: the following CLOCK_DEDICATED_ROUTE constraint will cause a warning in place similar
# to the following:
#   WARNING:Place:1402 - A clock IOB / PLL clock component pair have been found that are not
#   placed at an optimal clock IOB / PLL site pair.
# This warning can be ignored.  See the Users Guide for more information.

##set_property CLOCK_DEDICATED_ROUTE BACKBONE [get_nets qdriip_sys_clk_p]
set_property CLOCK_DEDICATED_ROUTE BACKBONE [get_pins -hierarchical *pll*CLKIN1]
          
########################################### SRAM Clock Pinout Constraints ###########################################

# PadFunction: IO_L14P_T2_SRCC_33 
set_property IOSTANDARD LVDS [get_ports {qdriip_sys_clk_p}]
set_property LOC AA10 [get_ports {qdriip_sys_clk_p}]

# PadFunction: IO_L14N_T2_SRCC_33 
set_property IOSTANDARD LVDS [get_ports {qdriip_sys_clk_n}]
set_property LOC AB10 [get_ports {qdriip_sys_clk_n}]

############################################# SRAM-A Pinout Constraints #############################################

# PadFunction: IO_L14N_T2_SRCC_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[0]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[0]}]
set_property LOC AC17 [get_ports {sram_a_qdriip_d[0]}]

# PadFunction: IO_L13N_T2_MRCC_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[1]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[1]}]
set_property LOC AD18 [get_ports {sram_a_qdriip_d[1]}]

# PadFunction: IO_L13P_T2_MRCC_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[2]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[2]}]
set_property LOC AC18 [get_ports {sram_a_qdriip_d[2]}]

# PadFunction: IO_L17N_T2_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[3]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[3]}]
set_property LOC AD19 [get_ports {sram_a_qdriip_d[3]}]

# PadFunction: IO_L17P_T2_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[4]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[4]}]
set_property LOC AC19 [get_ports {sram_a_qdriip_d[4]}]

# PadFunction: IO_L18P_T2_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[5]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[5]}]
set_property LOC AB19 [get_ports {sram_a_qdriip_d[5]}]

# PadFunction: IO_L16P_T2_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[6]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[6]}]
set_property LOC AA19 [get_ports {sram_a_qdriip_d[6]}]

# PadFunction: IO_L18N_T2_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[7]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[7]}]
set_property LOC AB20 [get_ports {sram_a_qdriip_d[7]}]

# PadFunction: IO_L16N_T2_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[8]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[8]}]
set_property LOC AA20 [get_ports {sram_a_qdriip_d[8]}]

# PadFunction: IO_L9N_T1_DQS_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[9]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[9]}]
set_property LOC Y16 [get_ports {sram_a_qdriip_d[9]}]

# PadFunction: IO_L12P_T1_MRCC_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[10]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[10]}]
set_property LOC AB16 [get_ports {sram_a_qdriip_d[10]}]

# PadFunction: IO_L10P_T1_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[11]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[11]}]
set_property LOC AB14 [get_ports {sram_a_qdriip_d[11]}]

# PadFunction: IO_L12N_T1_MRCC_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[12]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[12]}]
set_property LOC AC16 [get_ports {sram_a_qdriip_d[12]}]

# PadFunction: IO_L8P_T1_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[13]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[13]}]
set_property LOC AC14 [get_ports {sram_a_qdriip_d[13]}]

# PadFunction: IO_L9P_T1_DQS_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[14]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[14]}]
set_property LOC Y15 [get_ports {sram_a_qdriip_d[14]}]

# PadFunction: IO_L7N_T1_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[15]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[15]}]
set_property LOC AA15 [get_ports {sram_a_qdriip_d[15]}]

# PadFunction: IO_L10N_T1_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[16]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[16]}]
set_property LOC AB15 [get_ports {sram_a_qdriip_d[16]}]

# PadFunction: IO_L8N_T1_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_d[17]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_d[17]}]
set_property LOC AD14 [get_ports {sram_a_qdriip_d[17]}]

# PadFunction: IO_L13N_T2_MRCC_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[0]}]
set_property LOC AC11 [get_ports {sram_a_qdriip_q[0]}]

# PadFunction: IO_L17N_T2_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[1]}]
set_property LOC AD13 [get_ports {sram_a_qdriip_q[1]}]

# PadFunction: IO_L15N_T2_DQS_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[2]}]
set_property LOC AC12 [get_ports {sram_a_qdriip_q[2]}]

# PadFunction: IO_L15P_T2_DQS_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[3]}]
set_property LOC AB12 [get_ports {sram_a_qdriip_q[3]}]

# PadFunction: IO_L17P_T2_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[4]}]
set_property LOC AC13 [get_ports {sram_a_qdriip_q[4]}]

# PadFunction: IO_L16N_T2_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[5]}]
set_property LOC AA12 [get_ports {sram_a_qdriip_q[5]}]

# PadFunction: IO_L16P_T2_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[6]}]
set_property LOC AA13 [get_ports {sram_a_qdriip_q[6]}]

# PadFunction: IO_L18N_T2_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[7]}]
set_property LOC Y12 [get_ports {sram_a_qdriip_q[7]}]

# PadFunction: IO_L18P_T2_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[8]}]
set_property LOC Y13 [get_ports {sram_a_qdriip_q[8]}]

# PadFunction: IO_L23N_T3_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[9]}]
set_property LOC AF13 [get_ports {sram_a_qdriip_q[9]}]

# PadFunction: IO_L21P_T3_DQS_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[10]}]
set_property LOC AE12 [get_ports {sram_a_qdriip_q[10]}]

# PadFunction: IO_L21N_T3_DQS_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[11]}]
set_property LOC AF12 [get_ports {sram_a_qdriip_q[11]}]

# PadFunction: IO_L19P_T3_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[12]}]
set_property LOC AD11 [get_ports {sram_a_qdriip_q[12]}]

# PadFunction: IO_L20P_T3_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[13]}]
set_property LOC AD10 [get_ports {sram_a_qdriip_q[13]}]

# PadFunction: IO_L20N_T3_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[14]}]
set_property LOC AE10 [get_ports {sram_a_qdriip_q[14]}]

# PadFunction: IO_L24N_T3_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[15]}]
set_property LOC AF9 [get_ports {sram_a_qdriip_q[15]}]

# PadFunction: IO_L24P_T3_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[16]}]
set_property LOC AF10 [get_ports {sram_a_qdriip_q[16]}]

# PadFunction: IO_L22N_T3_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_q[17]}]
set_property LOC AF8 [get_ports {sram_a_qdriip_q[17]}]

# PadFunction: IO_L5N_T0_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[20]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[20]}]
set_property LOC AF20 [get_ports {sram_a_qdriip_sa[20]}]

# PadFunction: IO_L3P_T0_DQS_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[19]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[19]}]
set_property LOC AE18 [get_ports {sram_a_qdriip_sa[19]}]

# PadFunction: IO_L1N_T0_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[18]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[18]}]
set_property LOC AF17 [get_ports {sram_a_qdriip_sa[18]}]

# PadFunction: IO_L4P_T0_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[17]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[17]}]
set_property LOC AD15 [get_ports {sram_a_qdriip_sa[17]}]

# PadFunction: IO_L2N_T0_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[16]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[16]}]
set_property LOC AF15 [get_ports {sram_a_qdriip_sa[16]}]

# PadFunction: IO_L2P_T0_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[15]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[15]}]
set_property LOC AF14 [get_ports {sram_a_qdriip_sa[15]}]

# PadFunction: IO_L5P_T0_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[14]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[14]}]
set_property LOC AF19 [get_ports {sram_a_qdriip_sa[14]}]

# PadFunction: IO_L1P_T0_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[13]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[13]}]
set_property LOC AE17 [get_ports {sram_a_qdriip_sa[13]}]

# PadFunction: IO_L6N_T0_VREF_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[12]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[12]}]
set_property LOC AE16 [get_ports {sram_a_qdriip_sa[12]}]

# PadFunction: IO_L4N_T0_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[11]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[11]}]
set_property LOC AE15 [get_ports {sram_a_qdriip_sa[11]}]

# PadFunction: IO_L3N_T0_DQS_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[10]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[10]}]
set_property LOC AF18 [get_ports {sram_a_qdriip_sa[10]}]

# PadFunction: IO_L19P_T3_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[9]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[9]}]
set_property LOC Y17 [get_ports {sram_a_qdriip_sa[9]}]

# PadFunction: IO_L6P_T0_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[8]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[8]}]
set_property LOC AD16 [get_ports {sram_a_qdriip_sa[8]}]

# PadFunction: IO_L19N_T3_VREF_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[7]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[7]}]
set_property LOC Y18 [get_ports {sram_a_qdriip_sa[7]}]

# PadFunction: IO_L20P_T3_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[6]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[6]}]
set_property LOC V16 [get_ports {sram_a_qdriip_sa[6]}]

# PadFunction: IO_L21P_T3_DQS_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[5]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[5]}]
set_property LOC W18 [get_ports {sram_a_qdriip_sa[5]}]

# PadFunction: IO_L22P_T3_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[4]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[4]}]
set_property LOC W15 [get_ports {sram_a_qdriip_sa[4]}]

# PadFunction: IO_L23N_T3_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[3]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[3]}]
set_property LOC V19 [get_ports {sram_a_qdriip_sa[3]}]

# PadFunction: IO_L21N_T3_DQS_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[2]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[2]}]
set_property LOC W19 [get_ports {sram_a_qdriip_sa[2]}]

# PadFunction: IO_L24P_T3_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[1]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[1]}]
set_property LOC V14 [get_ports {sram_a_qdriip_sa[1]}]

# PadFunction: IO_L24N_T3_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_sa[0]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_sa[0]}]
set_property LOC W14 [get_ports {sram_a_qdriip_sa[0]}]

# PadFunction: IO_L22N_T3_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_w_n}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_w_n}]
set_property LOC W16 [get_ports {sram_a_qdriip_w_n}]

# PadFunction: IO_L23P_T3_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_r_n}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_r_n}]
set_property LOC V18 [get_ports {sram_a_qdriip_r_n}]

# PadFunction: IO_L7P_T1_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_dll_off_n}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_dll_off_n}]
set_property LOC AA14 [get_ports {sram_a_qdriip_dll_off_n}]

# PadFunction: IO_L14P_T2_SRCC_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_bw_n[0]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_bw_n[0]}]
set_property LOC AB17 [get_ports {sram_a_qdriip_bw_n[0]}]

# PadFunction: IO_L11P_T1_SRCC_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_bw_n[1]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_a_qdriip_bw_n[1]}]
set_property LOC AA17 [get_ports {sram_a_qdriip_bw_n[1]}]

# PadFunction: IO_L13P_T2_MRCC_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_cq}]
set_property LOC AB11 [get_ports {sram_a_qdriip_cq}]

# PadFunction: IO_L15P_T2_DQS_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_k_p}]
set_property IOSTANDARD DIFF_HSTL_I_18 [get_ports {sram_a_qdriip_k_p}]
set_property LOC AD20 [get_ports {sram_a_qdriip_k_p}]

# PadFunction: IO_L15N_T2_DQS_32 
set_property SLEW FAST [get_ports {sram_a_qdriip_k_n}]
set_property IOSTANDARD DIFF_HSTL_I_18 [get_ports {sram_a_qdriip_k_n}]
set_property LOC AE20 [get_ports {sram_a_qdriip_k_n}]


# Map the QVLD pin to a terminated IBUF even though it is not used.
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_a_qdriip_qvld}]
set_property LOC AE13 [get_ports {sram_a_qdriip_qvld}]


############################################# SRAM-B Pinout Constraints #############################################

# PadFunction: IO_L14P_T2_SRCC_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[0]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[0]}]
set_property LOC AC4 [get_ports {sram_b_qdriip_d[0]}]

# PadFunction: IO_L18N_T2_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[1]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[1]}]
set_property LOC AD5 [get_ports {sram_b_qdriip_d[1]}]

# PadFunction: IO_L15N_T2_DQS_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[2]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[2]}]
set_property LOC AB5 [get_ports {sram_b_qdriip_d[2]}]

# PadFunction: IO_L13N_T2_MRCC_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[3]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[3]}]
set_property LOC AB4 [get_ports {sram_b_qdriip_d[3]}]

# PadFunction: IO_L15P_T2_DQS_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[4]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[4]}]
set_property LOC AA5 [get_ports {sram_b_qdriip_d[4]}]

# PadFunction: IO_L18P_T2_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[5]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[5]}]
set_property LOC AD6 [get_ports {sram_b_qdriip_d[5]}]

# PadFunction: IO_L13P_T2_MRCC_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[6]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[6]}]
set_property LOC AA4 [get_ports {sram_b_qdriip_d[6]}]

# PadFunction: IO_L16N_T2_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[7]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[7]}]
set_property LOC AC6 [get_ports {sram_b_qdriip_d[7]}]

# PadFunction: IO_L16P_T2_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[8]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[8]}]
set_property LOC AB6 [get_ports {sram_b_qdriip_d[8]}]

# PadFunction: IO_L10P_T1_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[9]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[9]}]
set_property LOC W1 [get_ports {sram_b_qdriip_d[9]}]

# PadFunction: IO_L10N_T1_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[10]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[10]}]
set_property LOC Y1 [get_ports {sram_b_qdriip_d[10]}]

# PadFunction: IO_L8N_T1_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[11]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[11]}]
set_property LOC V1 [get_ports {sram_b_qdriip_d[11]}]

# PadFunction: IO_L7N_T1_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[12]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[12]}]
set_property LOC Y2 [get_ports {sram_b_qdriip_d[12]}]

# PadFunction: IO_L8P_T1_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[13]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[13]}]
set_property LOC V2 [get_ports {sram_b_qdriip_d[13]}]

# PadFunction: IO_L11P_T1_SRCC_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[14]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[14]}]
set_property LOC AB2 [get_ports {sram_b_qdriip_d[14]}]

# PadFunction: IO_L7P_T1_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[15]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[15]}]
set_property LOC Y3 [get_ports {sram_b_qdriip_d[15]}]

# PadFunction: IO_L12P_T1_MRCC_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[16]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[16]}]
set_property LOC AA3 [get_ports {sram_b_qdriip_d[16]}]

# PadFunction: IO_L11N_T1_SRCC_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_d[17]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_d[17]}]
set_property LOC AC2 [get_ports {sram_b_qdriip_d[17]}]

# PadFunction: IO_L7N_T1_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[0]}]
set_property LOC AF7 [get_ports {sram_b_qdriip_q[0]}]

# PadFunction: IO_L7P_T1_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[1]}]
set_property LOC AE7 [get_ports {sram_b_qdriip_q[1]}]

# PadFunction: IO_L9N_T1_DQS_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[2]}]
set_property LOC AD8 [get_ports {sram_b_qdriip_q[2]}]

# PadFunction: IO_L9P_T1_DQS_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[3]}]
set_property LOC AC8 [get_ports {sram_b_qdriip_q[3]}]

# PadFunction: IO_L10N_T1_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[4]}]
set_property LOC AC7 [get_ports {sram_b_qdriip_q[4]}]

# PadFunction: IO_L12N_T1_MRCC_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[5]}]
set_property LOC AD9 [get_ports {sram_b_qdriip_q[5]}]

# PadFunction: IO_L11N_T1_SRCC_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[6]}]
set_property LOC AB9 [get_ports {sram_b_qdriip_q[6]}]

# PadFunction: IO_L10P_T1_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[7]}]
set_property LOC AB7 [get_ports {sram_b_qdriip_q[7]}]

# PadFunction: IO_L11P_T1_SRCC_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[8]}]
set_property LOC AA9 [get_ports {sram_b_qdriip_q[8]}]

# PadFunction: IO_L2N_T0_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[9]}]
set_property LOC V7 [get_ports {sram_b_qdriip_q[9]}]

# PadFunction: IO_L2P_T0_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[10]}]
set_property LOC V8 [get_ports {sram_b_qdriip_q[10]}]

# PadFunction: IO_L6P_T0_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[11]}]
set_property LOC V9 [get_ports {sram_b_qdriip_q[11]}]

# PadFunction: IO_L4N_T0_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[12]}]
set_property LOC Y7 [get_ports {sram_b_qdriip_q[12]}]

# PadFunction: IO_L4P_T0_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[13]}]
set_property LOC Y8 [get_ports {sram_b_qdriip_q[13]}]

# PadFunction: IO_L3P_T0_DQS_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[14]}]
set_property LOC W10 [get_ports {sram_b_qdriip_q[14]}]

# PadFunction: IO_L3N_T0_DQS_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[15]}]
set_property LOC W9 [get_ports {sram_b_qdriip_q[15]}]

# PadFunction: IO_L5N_T0_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[16]}]
set_property LOC Y10 [get_ports {sram_b_qdriip_q[16]}]

# PadFunction: IO_L5P_T0_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_q[17]}]
set_property LOC Y11 [get_ports {sram_b_qdriip_q[17]}]

# PadFunction: IO_L23P_T3_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[20]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[20]}]
set_property LOC AE6 [get_ports {sram_b_qdriip_sa[20]}]

# PadFunction: IO_L21P_T3_DQS_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[19]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[19]}]
set_property LOC AF5 [get_ports {sram_b_qdriip_sa[19]}]

# PadFunction: IO_L21N_T3_DQS_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[18]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[18]}]
set_property LOC AF4 [get_ports {sram_b_qdriip_sa[18]}]

# PadFunction: IO_L24P_T3_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[17]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[17]}]
set_property LOC AF3 [get_ports {sram_b_qdriip_sa[17]}]

# PadFunction: IO_L22N_T3_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[16]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[16]}]
set_property LOC AE2 [get_ports {sram_b_qdriip_sa[16]}]

# PadFunction: IO_L20P_T3_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[15]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[15]}]
set_property LOC AD1 [get_ports {sram_b_qdriip_sa[15]}]

# PadFunction: IO_L23N_T3_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[14]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[14]}]
set_property LOC AE5 [get_ports {sram_b_qdriip_sa[14]}]

# PadFunction: IO_L19P_T3_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[13]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[13]}]
set_property LOC AD4 [get_ports {sram_b_qdriip_sa[13]}]

# PadFunction: IO_L22P_T3_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[12]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[12]}]
set_property LOC AE3 [get_ports {sram_b_qdriip_sa[12]}]

# PadFunction: IO_L20N_T3_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[11]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[11]}]
set_property LOC AE1 [get_ports {sram_b_qdriip_sa[11]}]

# PadFunction: IO_L19N_T3_VREF_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[10]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[10]}]
set_property LOC AD3 [get_ports {sram_b_qdriip_sa[10]}]

# PadFunction: IO_L6N_T0_VREF_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[9]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[9]}]
set_property LOC W4 [get_ports {sram_b_qdriip_sa[9]}]

# PadFunction: IO_L24N_T3_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[8]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[8]}]
set_property LOC AF2 [get_ports {sram_b_qdriip_sa[8]}]

# PadFunction: IO_L3N_T0_DQS_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[7]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[7]}]
set_property LOC W5 [get_ports {sram_b_qdriip_sa[7]}]

# PadFunction: IO_L6P_T0_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[6]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[6]}]
set_property LOC V4 [get_ports {sram_b_qdriip_sa[6]}]

# PadFunction: IO_L5N_T0_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[5]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[5]}]
set_property LOC V6 [get_ports {sram_b_qdriip_sa[5]}]

# PadFunction: IO_L4N_T0_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[4]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[4]}]
set_property LOC W3 [get_ports {sram_b_qdriip_sa[4]}]

# PadFunction: IO_L5P_T0_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[3]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[3]}]
set_property LOC U7 [get_ports {sram_b_qdriip_sa[3]}]

# PadFunction: IO_L3P_T0_DQS_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[2]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[2]}]
set_property LOC W6 [get_ports {sram_b_qdriip_sa[2]}]

# PadFunction: IO_L2P_T0_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[1]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[1]}]
set_property LOC U2 [get_ports {sram_b_qdriip_sa[1]}]

# PadFunction: IO_L2N_T0_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_sa[0]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_sa[0]}]
set_property LOC U1 [get_ports {sram_b_qdriip_sa[0]}]

# PadFunction: IO_L4P_T0_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_w_n}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_w_n}]
set_property LOC V3 [get_ports {sram_b_qdriip_w_n}]

# PadFunction: IO_L1P_T0_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_r_n}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_r_n}]
set_property LOC U6 [get_ports {sram_b_qdriip_r_n}]

# PadFunction: IO_L17N_T2_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_dll_off_n}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_dll_off_n}]
set_property LOC Y5 [get_ports {sram_b_qdriip_dll_off_n}]

# PadFunction: IO_L14N_T2_SRCC_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_bw_n[0]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_bw_n[0]}]
set_property LOC AC3 [get_ports {sram_b_qdriip_bw_n[0]}]

# PadFunction: IO_L12N_T1_MRCC_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_bw_n[1]}]
set_property IOSTANDARD HSTL_I_18 [get_ports {sram_b_qdriip_bw_n[1]}]
set_property LOC AA2 [get_ports {sram_b_qdriip_bw_n[1]}]

# PadFunction: IO_L12P_T1_MRCC_33 
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_cq}]
set_property LOC AC9 [get_ports {sram_b_qdriip_cq}]

# PadFunction: IO_L9P_T1_DQS_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_k_p}]
set_property IOSTANDARD DIFF_HSTL_I_18 [get_ports {sram_b_qdriip_k_p}]
set_property LOC AB1 [get_ports {sram_b_qdriip_k_p}]

# PadFunction: IO_L9N_T1_DQS_34 
set_property SLEW FAST [get_ports {sram_b_qdriip_k_n}]
set_property IOSTANDARD DIFF_HSTL_I_18 [get_ports {sram_b_qdriip_k_n}]
set_property LOC AC1 [get_ports {sram_b_qdriip_k_n}]


# Map the QVLD pin to a terminated IBUF even though it is not used.
set_property IOSTANDARD HSTL_I_DCI_18 [get_ports {sram_b_qdriip_qvld}]
set_property LOC AA7 [get_ports {sram_b_qdriip_qvld}]


############################################ Common Physical Constraints ############################################

# PHY reference block for I/O bank 33.
set_property LOC PHASER_REF_X1Y1 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/*/qdr_rld_phy_4lanes_0.u_qdr_rld_phy_4lanes/phaser_ref_i}]

# MMCM/PLL block for I/O bank 33.
set_property LOC PLLE2_ADV_X1Y1 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/u_infrastructure/plle2_i}]
set_property LOC MMCME2_ADV_X1Y1 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/u_infrastructure/gen_mmcm.mmcm_i}]

set_multicycle_path -through [get_pins -filter {NAME =~ */OSERDESRST} -of [get_cells -hier -filter {REF_NAME == PHASER_OUT_PHY}]] -setup 2 -start
set_multicycle_path -through [get_pins -filter {NAME =~ */OSERDESRST} -of [get_cells -hier -filter {REF_NAME == PHASER_OUT_PHY}]] -hold 1 -start

############################################ SRAM-A Physical Constraints ############################################

# PHY reference and control blocks for I/O bank 32.
set_property LOC PHY_CONTROL_X1Y0 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/*/qdr_rld_phy_4lanes_1.u_qdr_rld_phy_4lanes/PHY_CONTROL_INST.phy_control_i}]
set_property LOC PHASER_REF_X1Y0 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/*/qdr_rld_phy_4lanes_1.u_qdr_rld_phy_4lanes/phaser_ref_i}]

# PHY phaser output and FIFO blocks for I/O bank 32.
set_property LOC PHASER_OUT_PHY_X1Y3 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/*/qdr_rld_phy_4lanes_1.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_D.qdr_rld_byte_lane_D/PHASER_OUT_inst.phaser_out}]
set_property LOC PHASER_OUT_PHY_X1Y0 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/*/qdr_rld_phy_4lanes_1.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_A.qdr_rld_byte_lane_A/PHASER_OUT_inst.phaser_out}]
set_property LOC PHASER_OUT_PHY_X1Y2 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/*/qdr_rld_phy_4lanes_1.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_C.qdr_rld_byte_lane_C/PHASER_OUT_inst.phaser_out}]
set_property LOC PHASER_OUT_PHY_X1Y1 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/*/qdr_rld_phy_4lanes_1.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_B.qdr_rld_byte_lane_B/PHASER_OUT_inst.phaser_out}]

set_property LOC OUT_FIFO_X1Y3 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/*/qdr_rld_phy_4lanes_1.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_D.qdr_rld_byte_lane_D/out_fifo_inst.out_fifo}]
set_property LOC OUT_FIFO_X1Y0 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/*/qdr_rld_phy_4lanes_1.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_A.qdr_rld_byte_lane_A/out_fifo_inst.out_fifo}]
set_property LOC OUT_FIFO_X1Y2 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/*/qdr_rld_phy_4lanes_1.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_C.qdr_rld_byte_lane_C/out_fifo_inst.out_fifo}]
set_property LOC OUT_FIFO_X1Y1 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/*/qdr_rld_phy_4lanes_1.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_B.qdr_rld_byte_lane_B/out_fifo_inst.out_fifo}]

# PHY phaser input and FIFO blocks for I/O bank 33.
set_property LOC PHASER_IN_PHY_X1Y5 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/*/qdr_rld_phy_4lanes_0.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_B.qdr_rld_byte_lane_B/PHASER_IN_inst.phaser_in}]
set_property LOC PHASER_IN_PHY_X1Y4 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/*/qdr_rld_phy_4lanes_0.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_A.qdr_rld_byte_lane_A/PHASER_IN_inst.phaser_in}]

set_property LOC IN_FIFO_X1Y5 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/*/qdr_rld_phy_4lanes_0.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_B.qdr_rld_byte_lane_B/in_fifo_inst.in_fifo}]
set_property LOC IN_FIFO_X1Y4 [get_cells -hier -filter {NAME =~ */SRAM_A_MIG/*/qdr_rld_phy_4lanes_0.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_A.qdr_rld_byte_lane_A/in_fifo_inst.in_fifo}]


############################################ SRAM-B Physical Constraints ############################################

# PHY reference and control blocks for I/O bank 34.
set_property LOC PHY_CONTROL_X1Y2 [get_cells -hier -filter {NAME =~ */SRAM_B_MIG/*/qdr_rld_phy_4lanes_0.u_qdr_rld_phy_4lanes/PHY_CONTROL_INST.phy_control_i}]
set_property LOC PHASER_REF_X1Y2 [get_cells -hier -filter {NAME =~ */SRAM_B_MIG/*/qdr_rld_phy_4lanes_0.u_qdr_rld_phy_4lanes/PHY_CONTROL_INST.phaser_ref_i}]

# PHY phaser output and FIFO blocks for I/O bank 34.
set_property LOC PHASER_OUT_PHY_X1Y11 [get_cells -hier -filter {NAME =~ */SRAM_B_MIG/*/qdr_rld_phy_4lanes_0.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_D.qdr_rld_byte_lane_D/PHASER_OUT_inst.phaser_out}]
set_property LOC PHASER_OUT_PHY_X1Y8 [get_cells -hier -filter {NAME =~ */SRAM_B_MIG/*/qdr_rld_phy_4lanes_0.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_A.qdr_rld_byte_lane_A/PHASER_OUT_inst.phaser_out}]
set_property LOC PHASER_OUT_PHY_X1Y10 [get_cells -hier -filter {NAME =~ */SRAM_B_MIG/*/qdr_rld_phy_4lanes_0.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_C.qdr_rld_byte_lane_C/PHASER_OUT_inst.phaser_out}]
set_property LOC PHASER_OUT_PHY_X1Y9 [get_cells -hier -filter {NAME =~ */SRAM_B_MIG/*/qdr_rld_phy_4lanes_0.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_B.qdr_rld_byte_lane_B/PHASER_OUT_inst.phaser_out}]

set_property LOC OUT_FIFO_X1Y11 [get_cells -hier -filter {NAME =~ */SRAM_B_MIG/*/qdr_rld_phy_4lanes_0.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_D.qdr_rld_byte_lane_D/out_fifo_inst.out_fifo}]
set_property LOC OUT_FIFO_X1Y8 [get_cells -hier -filter {NAME =~ */SRAM_B_MIG/*/qdr_rld_phy_4lanes_0.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_A.qdr_rld_byte_lane_A/out_fifo_inst.out_fifo}]
set_property LOC OUT_FIFO_X1Y10 [get_cells -hier -filter {NAME =~ */SRAM_B_MIG/*/qdr_rld_phy_4lanes_0.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_C.qdr_rld_byte_lane_C/out_fifo_inst.out_fifo}]
set_property LOC OUT_FIFO_X1Y9 [get_cells -hier -filter {NAME =~ */SRAM_B_MIG/*/qdr_rld_phy_4lanes_0.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_B.qdr_rld_byte_lane_B/out_fifo_inst.out_fifo}]

# PHY phaser input and FIFO blocks for I/O bank 34.
set_property LOC PHASER_IN_PHY_X1Y7 [get_cells -hier -filter {NAME =~ */SRAM_B_MIG/*/qdr_rld_phy_4lanes_1.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_D.qdr_rld_byte_lane_D/PHASER_IN_inst.phaser_in}]
set_property LOC PHASER_IN_PHY_X1Y6 [get_cells -hier -filter {NAME =~ */SRAM_B_MIG/*/qdr_rld_phy_4lanes_1.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_C.qdr_rld_byte_lane_C/PHASER_IN_inst.phaser_in}]

set_property LOC IN_FIFO_X1Y7 [get_cells -hier -filter {NAME =~ */SRAM_B_MIG/*/qdr_rld_phy_4lanes_1.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_D.qdr_rld_byte_lane_D/in_fifo_inst.in_fifo}]
set_property LOC IN_FIFO_X1Y6 [get_cells -hier -filter {NAME =~ */SRAM_B_MIG/*/qdr_rld_phy_4lanes_1.u_qdr_rld_phy_4lanes/qdr_rld_byte_lane_C.qdr_rld_byte_lane_C/in_fifo_inst.in_fifo}]
