#!/bin/sh
#-*-tcl-*-

set num_arg $argc
set arg_error 0
set model 0 
set adc 0
set dac 0

global env tcl_platform

puts "Red Rapids SigFPGA Simulation"

# Detect if arguments can be recovered from argv
set no_argv 1
if {[llength $argv]>0} {
  set no_argv 0      
  foreach arg $argv {
    # check -do or -gui is not found
    if {([regexp {^-do.*} $arg]==1) || ([regexp {^-gui.*} $arg]==1)} {
      set no_argv 1                
    }
  }
}

if {$num_arg==0} {
  puts "-----------------------------------------------------------------------"
  puts "Usage: <scriptname> M372|M376|M377|M378 "
  puts ""
  puts " A model must be selected for simulation. "   
  puts "-----------------------------------------------------------------------"
  quit 1
}

for {set i 0} {$i<1} {incr i} {
  if {$no_argv==1} {
    set arg $1
    shift
  } else {
    set arg [lindex $argv $i]
  }
	# Model
	if {[regexp {372.*} $arg]==1} { set model 372 }
	if {[regexp {376.*} $arg]==1} { set model 376 }
	if {[regexp {377.*} $arg]==1} { set model 377 }
	if {[regexp {378.*} $arg]==1} { set model 378 }
}                   

if {($model!=372) && ($model!=376) && ($model!=377) && ($model!=378)} {
  set arg_error 1   
  puts "-----------------------------------------------------------------------"
  puts "  ERROR: Model number is uncorrect!"
}

if {$arg_error==1} {
  puts "-----------------------------------------------------------------------"
  puts "Usage: <scriptname> M372|M376|M377|M378 "
  puts ""
  puts " Please select a valid model for simulation. "   
  puts "-----------------------------------------------------------------------"
  quit 1
}


########################################################################
#
# Define path
#
########################################################################

puts "-----------------------------------------------------------------------"
puts "		Map Xilinx Libraries		"
puts "-----------------------------------------------------------------------"
do map_libraries_v1.do

switch $model {
	372 {
	vcom  -quiet -work work -2008 "../../../SigFPGA/M372/sources/model_specific_package_v3.vhd"
	vcom  -quiet -work work -2008 "../../../SigFPGA/M372/sources/tb_package_v0.vhd"
	set adc 2
	set dac 2
	}
	376 {
	vcom  -quiet -work work -2008 "../../../SigFPGA/M376/sources/model_specific_package_v3.vhd"
	vcom  -quiet -work work -2008 "../../../SigFPGA/M376/sources/tb_package_v0.vhd"
	set adc 2
	set dac 0
	}
	377 {
	vcom  -quiet -work work -2008 "../../../SigFPGA/M377/sources/model_specific_package_v3.vhd"
	vcom  -quiet -work work -2008 "../../../SigFPGA/M377/sources/tb_package_v0.vhd"
	set adc 4
	set dac 0
	}
	378 {
	vcom  -quiet -work work -2008 "../../../SigFPGA/M378/sources/model_specific_package_v0.vhd"
	vcom  -quiet -work work -2008 "../../../SigFPGA/M378/sources/tb_package_v0.vhd"
	set adc 8
	set dac 0
	}
	default {
		puts "-----------------------------------------------------------------------"
		puts "  ERROR: Model number is uncorrect!"
  		puts "	$model does not match any of the model choices." 
  		puts "-----------------------------------------------------------------------"
	}
}


puts "-----------------------------------------------------------------------"
puts "		Compile Design Files		"
puts "-----------------------------------------------------------------------"
do compile_files_v4.do

puts "-----------------------------------------------------------------------"
puts "  Starting Signal FPGA simulation!	"
puts "-----------------------------------------------------------------------"

vsim -t ps -L unisims_ver -L unimacro_ver -L secureip work.sigfpga_tb(structural) work.glbl

# Load waveform
puts "..loading waveform"
log -r /*
if {($adc > 0) && ($dac > 0)} {
	do xcvr_wave.do
} elseif {($adc == 0) && ($dac < 0)} {
	do tx_wave.do
} else {
	do rx_wave.do
}
if {$dac > 0} {
	puts "..load DAC data buffers"
	set axi_width [examine -decimal /model_specific_package/AXI_WIDTH]
	if {$axi_width == "32'd64"} {
		for {set j [expr {$adc+1}]} {$j <= [expr {$adc+$dac}]} {incr j} {
			mem load -i tx_64data.txt /sigfpga_tb/XCVR/PCI/TX_CH/TX($j\)/STATE_MACHINE/txdata
		}
	} elseif {$axi_width == "32'd128"} {
		for {set j [expr {$adc+1}]} {$j <= [expr {$adc+$dac}]} {incr j} {
			mem load -i tx_128data.txt /sigfpga_tb/XCVR/PCI/TX_CH/TX($j\)/STATE_MACHINE/txdata
		}
	}
}

puts "..start simulation"
set simstatus 0

puts "Waiting for simulation to finish!"
while {$simstatus != 1} {
	set simstatus [examine sim_status]
	run 1000 ns
}


# Save RX memory into a .txt file 
for {set i 1} {$i <= $adc} {incr i} {
	mem save -o rx$i\_dmadata.txt -f hex -noaddress -wordsperline 1 /sigfpga_tb/XCVR/PCI/RX_CH/RX($i)/STATE_MACHINE/rddata
}

puts ".. done"


